@@ -20,7 +20,7 @@
 # IN THE SOFTWARE.
 
 __author__ = 'Niklas Rosenstein <rosensteinniklas@gmail.com>'
-__version__ = '1.3.0'
+__version__ = '2.0.0'
 __all__ = [
   'Location',
   'Decoration',
@@ -29,8 +29,11 @@ __all__ = [
   'ApiObject',
   'Indirection',
   'HasMembers',
-  'Data',
+  'VariableSemantic',
+  'Variable',
+  'FunctionSemantic',
   'Function',
+  'ClassSemantic',
   'Class',
   'Module',
   'load_module',
@@ -38,7 +41,6 @@ __all__ = [
   'dump_module',
   'filter_visit',
   'visit',
-  'ReverseMap',
   'get_member',
 ]
 
@@ -51,7 +53,6 @@ import typing as t
 import typing_extensions as te
 import weakref
 
-import deprecated
 import databind.json
 from databind.core.annotations import alias, union
 
@@ -62,7 +63,7 @@ class Location:
   Represents the location of an #ApiObject by a filename and line number.
   """
 
-  filename: t.Optional[str]
+  filename: str
   lineno: int
 
   #: If the location of an entity spans over multiple lines, it can be indicated by specifying at
@@ -70,28 +71,21 @@ class Location:
   endlineno: t.Optional[int] = None
 
 
-@dataclasses.dataclass(init=False, frozen=True)
-class Docstring(str):
+@dataclasses.dataclass
+class Docstring:
   """
   Represents a docstring for an #APIObject, i.e. it's content and location. This class is a subclass of `str`
   for backwards compatibility reasons. Use the #content property to access the docstring content over the
   #Docstring value directory.
-
-  Added in `1.1.0`.
   """
 
   #: The location of where the docstring is defined.
-  location: t.Optional[Location]
+  location: Location = dataclasses.field(repr=False)
 
   #: The content of the docstring. While the #Docstring class is a subclass of `str` and holds
   #: the same value as *content*, using the #content property should be preferred as the inheritance
   #: from the `str` class may be removed in future versions.
-  content: str = t.cast(str, property(lambda self: str(self)))
-
-  def __new__(cls, content: str, location: t.Optional[Location]) -> 'Docstring':
-    obj = super().__new__(cls, content)
-    obj.__dict__['location'] = location
-    return obj
+  content: str
 
 
 @dataclasses.dataclass
@@ -100,6 +94,9 @@ class Decoration:
   Represents a decorator on a #Class or #Function.
   """
 
+  #: The location of the decoration in the source code.
+  location: Location = dataclasses.field(repr=False)
+
   #: The name of the decorator (i.e. the text between the `@` and `(`). In languages that support it,
   #: this may be a piece of code.
   name: str
@@ -109,9 +106,6 @@ class Decoration:
   #: For backwards compatibility, loaders may populate both the #args and #arglist fields.
   args: t.Optional[str] = None
 
-  #: The location of the decoration in the source code.
-  location: t.Optional[Location] = None
-
   #: Decorator arguments, one item per argument. For keyword arguments, the keyword name and equals
   #: sign preceed the argument value expression code.
   arglist: t.Optional[t.List[str]] = None
@@ -159,6 +153,9 @@ class Argument:
   Type.KeywordOnly = Type.KEYWORD_ONLY
   Type.KeywordRemainder = Type.KEYWORD_REMAINDER
 
+  #: The location of the argument in the source code.
+  location: Location = dataclasses.field(repr=False)
+
   #: The name of the argument.
   name: str
 
@@ -175,9 +172,6 @@ class Argument:
   #: The default value of the argument as a code string.
   default_value: t.Optional[str] = None
 
-  #: The location of the argument in the source code.
-  location: t.Optional[Location] = None
-
 
 @dataclasses.dataclass
 class ApiObject:
@@ -186,15 +180,15 @@ class ApiObject:
   be that a variable/constant, function, class or module.
   """
 
+  #: The location of the API object, i.e. where it is sourced from/defined in the code.
+  location: Location = dataclasses.field(repr=False)
+
   #: The name of the entity. This is usually relative to the respective parent of the entity,
   #: as opposed to it's fully qualified name/absolute name. However, that is more of a
   #: recommendation than rule. For example the #docspec_python loader by default returns
   #: #Module objects with their full module name (and does not create a module hierarchy).
   name: str
 
-  #: The location of the API object, i.e. where it is sourced from/defined in the code.
-  location: t.Optional[Location] = dataclasses.field(repr=False)
-
   #: The documentation string of the API object.
   docstring: t.Optional[Docstring] = dataclasses.field(repr=False)
 
@@ -250,25 +244,28 @@ class ApiObject:
     self.parent = parent
 
 
+class VariableSemantic(enum.Enum):
+  """
+  A list of well-known properties and behaviour that can be attributed to a variable/constant.
+  """
+
+  #: The #Variable object is an instance variable of a class.
+  INSTANCE_VARIABLE = 0
+
+  #: The #Variable object is a static variable of a class.
+  CLASS_VARIABLE = 1
+
+  #: The #Variable object represents a constant value.
+  CONSTANT = 2
+
+
 @dataclasses.dataclass
-class Data(ApiObject):
+class Variable(ApiObject):
   """
   Represents a variable assignment (e.g. for global variables (often used as constants) or class members).
   """
 
-  class Semantic(enum.Enum):
-    """
-    A list of well-known properties and behaviour that can be attributed to a variable/constant.
-    """
-
-    #: The #Data object is an instance variable of a class.
-    INSTANCE_VARIABLE = 0
-
-    #: The #Data object is a static variable of a class.
-    CLASS_VARIABLE = 1
-
-    #: The #Data object represents a constant value.
-    CONSTANT = 2
+  Semantic: t.ClassVar = VariableSemantic
 
   #: The datatype associated with the assignment as code.
   datatype: t.Optional[str] = None
@@ -276,12 +273,12 @@ class Data(ApiObject):
   #: The value of the variable as code.
   value: t.Optional[str] = None
 
-  #: A list of language-specific modifiers that were used to declare this #Data object.
+  #: A list of language-specific modifiers that were used to declare this #Variable object.
   modifiers: t.List[str] = dataclasses.field(default_factory=list)
 
-  #: A list of hints that express semantics of this #Data object which are not otherwise
+  #: A list of hints that express semantics of this #Variable object which are not otherwise
   #: derivable from the context.
-  semantic_hints: t.List[Semantic] = dataclasses.field(default_factory=list)
+  semantic_hints: t.List[VariableSemantic] = dataclasses.field(default_factory=list)
 
 
 @dataclasses.dataclass
@@ -294,6 +291,42 @@ class Indirection(ApiObject):
   target: str
 
 
+class FunctionSemantic(enum.Enum):
+  """
+  A list of well-known properties and behaviour that can be attributed to a function.
+  """
+
+  #: The function is abstract.
+  ABSTRACT = 0
+
+  #: The function is final.
+  FINAL = 1
+
+  #: The function is a coroutine.
+  COROUTINE = 2
+
+  #: The function does not return.
+  NO_RETURN = 3
+
+  #: The function is an instance method.
+  INSTANCE_METHOD = 4
+
+  #: The function is a classmethod.
+  CLASS_METHOD = 5
+
+  #: The function is a staticmethod.
+  STATIC_METHOD = 6
+
+  #: The function is a property getter.
+  PROPERTY_GETTER = 7
+
+  #: The function is a property setter.
+  PROPERTY_SETTER = 8
+
+  #: The function is a property deleter.
+  PROPERTY_DELETER = 9
+
+
 @dataclasses.dataclass
 class Function(ApiObject):
   """
@@ -302,40 +335,7 @@ class Function(ApiObject):
   `@property`, `@classmethod` or `@staticmethod`?).
   """
 
-  class Semantic(enum.Enum):
-    """
-    A list of well-known properties and behaviour that can be attributed to a function.
-    """
-
-    #: The function is abstract.
-    ABSTRACT = 0
-
-    #: The function is final.
-    FINAL = 1
-
-    #: The function is a coroutine.
-    COROUTINE = 2
-
-    #: The function does not return.
-    NO_RETURN = 3
-
-    #: The function is an instance method.
-    INSTANCE_METHOD = 4
-
-    #: The function is a classmethod.
-    CLASS_METHOD = 5
-
-    #: The function is a staticmethod.
-    STATIC_METHOD = 6
-
-    #: The function is a property getter.
-    PROPERTY_GETTER = 7
-
-    #: The function is a property setter.
-    PROPERTY_SETTER = 8
-
-    #: The function is a property deleter.
-    PROPERTY_DELETER = 9
+  Semantic: t.ClassVar = FunctionSemantic
 
   #: A list of modifiers used in the function definition. For example, the only valid modifier in
   #: Python is "async".
@@ -351,7 +351,7 @@ class Function(ApiObject):
   decorations: t.Optional[t.List[Decoration]]
 
   #: A list of hints that describe the object.
-  semantic_hints: t.List[Semantic] = dataclasses.field(default_factory=list)
+  semantic_hints: t.List[FunctionSemantic] = dataclasses.field(default_factory=list)
 
 
 class HasMembers(ApiObject):
@@ -368,28 +368,31 @@ class HasMembers(ApiObject):
       member.sync_hierarchy(self)
 
 
+class ClassSemantic(enum.Enum):
+  """
+  A list of well-known properties and behaviour that can be attributed to a class.
+  """
+
+  #: The class describes an interface.
+  INTERFACE = 0
+
+  #: The class is abstract.
+  ABSTRACT = 1
+
+  #: The class is final.
+  FINAL = 2
+
+  #: The class is an enumeration.
+  ENUM = 3
+
+
 @dataclasses.dataclass
 class Class(HasMembers):
   """
   Represents a class definition.
   """
 
-  class Semantic(enum.Enum):
-    """
-    A list of well-known properties and behaviour that can be attributed to a class.
-    """
-
-    #: The class describes an interface.
-    INTERFACE = 0
-
-    #: The class is abstract.
-    ABSTRACT = 1
-
-    #: The class is final.
-    FINAL = 2
-
-    #: The class is an enumeration.
-    ENUM = 3
+  Semantic: t.ClassVar = ClassSemantic
 
   #: The metaclass used in the class definition as a code string.
   metaclass: t.Optional[str]
@@ -404,11 +407,11 @@ class Class(HasMembers):
   #: that class unless some information about the #Function indicates otherwise.
   members: t.List['_MemberType']
 
-  #: A list of language-specific modifiers that were used to declare this #Data object.
+  #: A list of language-specific modifiers that were used to declare this #Variable object.
   modifiers: t.List[str] = dataclasses.field(default_factory=list)
 
   #: A list of hints that describe the object.
-  semantic_hints: t.List[Semantic] = dataclasses.field(default_factory=list)
+  semantic_hints: t.List[ClassSemantic] = dataclasses.field(default_factory=list)
 
 
 @dataclasses.dataclass
@@ -423,16 +426,16 @@ class Module(HasMembers):
   members: t.List['_ModuleMemberType']
 
 
-_Members = t.Union[Data, Function, Class, Indirection]
+_Members = t.Union[Variable, Function, Class, Indirection]
 _MemberType = te.Annotated[
   _Members,
-  union({ 'data': Data, 'function': Function, 'class': Class, 'indirection': Indirection }, style=union.Style.flat)]
+  union({ 'data': Variable, 'function': Function, 'class': Class, 'indirection': Indirection }, style=union.Style.flat)]
 
 
-_ModuleMembers = t.Union[Data, Function, Class, Module, Indirection]
+_ModuleMembers = t.Union[Variable, Function, Class, Module, Indirection]
 _ModuleMemberType = te.Annotated[
   _ModuleMembers,
-  union({ 'data': Data, 'function': Function, 'class': Class, 'module': Module, 'indirection': Indirection },
+  union({ 'data': Variable, 'function': Function, 'class': Class, 'module': Module, 'indirection': Indirection },
     style=union.Style.flat)]
 
 
@@ -538,10 +541,10 @@ def dump_module(
 
 
 def filter_visit(
-  objects: t.List[ApiObject],
+  objects: t.MutableSequence[ApiObject],
   predicate: t.Callable[[ApiObject], bool],
   order: str = 'pre',
-) -> t.List[ApiObject]:
+) -> t.MutableSequence[ApiObject]:
   """
   Visits all *objects* recursively, applying the *predicate* in the specified *order*. If
   the predicate returrns #False, the object will be removed from it's containing list.
@@ -578,7 +581,7 @@ def filter_visit(
 
 
 def visit(
-  objects: t.List[ApiObject],
+  objects: t.Sequence[ApiObject],
   func: t.Callable[[ApiObject], t.Any],
   order: str = 'pre',
 ) -> None:
@@ -586,34 +589,17 @@ def visit(
   Visits all *objects*, applying *func* in the specified *order*.
   """
 
-  filter_visit(objects, (lambda obj: func(obj) or True), order)
-
-
-@deprecated.deprecated(
-  version='1.1.0',
-  reason='#docspec.ReverseMap is deprecated and will be removed in a future version, use #ApiObject.parent '
-         'and #ApiObject.path instead')
-class ReverseMap:
-  """
-  Reverse map for finding the parent of an #ApiObject.
-
-  @deprecated 1.1.0 -- Use #ApiObject.parent and #ApiObject.path instead of #get_parent() and #path().
-  """
-
-  def __init__(self, modules: t.List[Module]) -> None:
-    pass  # don't actually need the modules since 1.1.0
-
-  def get_parent(self, obj: ApiObject) -> t.Optional[ApiObject]:
-    return obj.parent
-
-  def path(self, obj: ApiObject) -> t.List[ApiObject]:
-    return obj.path
+  filter_visit(
+    t.cast(t.MutableSequence[ApiObject], objects),  # Sequence does not get mutated in this call
+    (lambda obj: func(obj) or True),
+    order,
+  )
 
 
 def get_member(obj: ApiObject, name: str) -> t.Optional[ApiObject]:
   """
   Generic function to retrieve a member from an API object. This will always return #None for
-  objects that don't support members (eg. #Function and #Data).
+  objects that don't support members (eg. #Function and #Variable).
   """
 
   if isinstance(obj, HasMembers):
